/************************************************************************/
/*                                                                      */
/*                      I2C Sniffer					*/
/*                                                                      */
/*              Author: Peter Dannegger                                 */
/*                                                                      */
/************************************************************************/
#include "main.h"
#include "uart.h"
#include "i2csniff.h"

void init_i2c( void )
{
  SCL 		= 1;
  SCL_DDR 	= 1;						// force busy waiting
  SDA 		= 1;
  SDA_DDR 	= 0;						// listen only, no ACK, no data
  USICR = 1<<USIWM1^1<<USIWM0;			// I2C, no counter
}


void putnibble( u8 j )
{
  j &= 0x0F;
  if( j > 9 )
    j += 'A' - '0' - 10;
  uart_putc( j + '0' );
}

void i2c_sniff( void )
{
  u8 get_ack 		= 0;
  u8 START_Received = 0;
  u8 Device_Address = 0;
  u8 Buffer			= 0;

  for(;;){
  	Buffer = USISR;

    if( Buffer & 1<<USIPF ){				// STOP received
      USICR = 1<<USIWM1^1<<USIWM0;		// I2C, no counter
      USISR = 1<<USIPF;					// clear USISIF
      uart_puts( STOP LINEFEED );
      continue;
    }

    if( Buffer & 1<<USISIF ){			// START received
      while( SCL_PIN );					// wait until SCL = low
      USICR = 1<<USIWM1^1<<USIWM0^1<<USICS1^1<<USICS0;
										// I2C, get data on neg. edge
      USISR = 1<<USISIF^1<<USIOIF^(0x0F & -16);
										// clear USISIF, count 16 edges
      get_ack = 0;
	  START_Received = 1;
	  uart_putc( START );
      continue;
    }

    if( Buffer & 1<<USIOIF ){				// Data / ACK received
		Buffer = USIDR;
		if( ++get_ack & 1 ){				// toggle read data / ACK
			USISR = 1<<USIOIF^(0x0F & -2);	// count 2 edges
			if ( START_Received ) {
				Device_Address = Buffer >> 1;
				putnibble( Device_Address >> 4 );
				putnibble( Device_Address );
				uart_putc( (Buffer & 1) ? READ : WRITE );
				START_Received = 0;
			}else{
				putnibble( Buffer >> 4 );
				putnibble( Buffer );
			}
		}else{
			USISR = 1<<USIOIF^(0x0F & -16);	// count 16 edges
			uart_putc( (Buffer & 1) ? NACK : ACK );
		}
    }	
  }
}
